/*
	Copyright (c) 2004 Jonatan Kronqvist

	Permission is hereby granted, free of charge, to any person obtaining a
	copy of this software and associated documentation files (the "Software"),
	to deal in the Software without restriction, including without limitation
	the rights to use, copy, modify, merge, publish, distribute, sublicense,
	and/or sell copies of the Software, and to permit persons to whom the
	Software is furnished to do so, subject to the following conditions:

	The above copyright notice and this permission notice shall be included in
	all copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
	OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
	FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
	IN THE SOFTWARE.
*/


#include "buffer.h"


buffer::buffer()
{
	m_buf = new char[1024];	// default size;
	m_size = 1024;
	clear();
}

buffer::buffer(int size)
{
	m_buf = new char[size];
	m_size = size;
	clear();
}

buffer::~buffer()
{
	SAFE_DELETE(m_buf);
}

bool buffer::append(int integer)
{
	if((m_len+sizeof(int)) > m_size) return false;

	memcpy(m_buf+m_len, &integer, sizeof(int));
	m_len += sizeof(int);

	return true;
}

bool buffer::append(unsigned short shortint)
{
	if((m_len+sizeof(unsigned short)) > m_size) return false;

	memcpy(m_buf+m_len, &shortint, sizeof(unsigned short));
	m_len += sizeof(unsigned short);

	return true;
}


bool buffer::append(char byte)
{
	if((m_len+sizeof(char)) > m_size) return false;

	memcpy(m_buf+m_len, &byte, sizeof(char));
	m_len += sizeof(char);

	return true;
}

bool buffer::append(unsigned char byte)
{
	if((m_len+sizeof(unsigned char)) > m_size) return false;

	memcpy(m_buf+m_len, &byte, sizeof(unsigned char));
	m_len += sizeof(unsigned char);

	return true;
}

bool buffer::append(string8 str)
{
	if((m_len+str.length()) > m_size) return false;

	memcpy(m_buf+m_len, str.get_ptr(), str.length());
	m_len += str.length();

	return true;
}

int buffer::length()
{
	return m_len;
}

int buffer::size()
{
	return m_size;
}

void buffer::clear()
{
	m_len = 0;
	memset(m_buf, 0, m_size);
}

char* buffer::get_ptr()
{
	return m_buf;
}